package cn.youle.pet.shop.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.util.CreditCodeUtil;
import cn.hutool.core.util.IdUtil;
import cn.youle.pet.shop.enums.ENUM_DELETE_TYPE;
import cn.youle.pet.shop.exception.PetShopException;
import cn.youle.pet.shop.pojo.dto.AdminsSaveDTO;
import cn.youle.pet.shop.pojo.dto.LoginReqDTO;
import cn.youle.pet.shop.pojo.entity.PetAdmins;
import cn.youle.pet.shop.mapper.PetAdminsMapper;
import cn.youle.pet.shop.pojo.vo.LoginResVO;
import cn.youle.pet.shop.service.IPetAdminsService;
import cn.youle.pet.shop.shiro.auth.TokenVO;
import cn.youle.pet.shop.utils.JWTToken;
import cn.youle.pet.shop.utils.JWTUtil;
import cn.youle.pet.shop.utils.VerifyCodeUtils;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.AuthenticationToken;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.time.Duration;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 管理员表 服务实现类
 * </p>
 *
 * @author wangrongli
 * @since 2023-09-11
 */
@Service
@Slf4j
public class PetAdminsServiceImpl extends ServiceImpl<PetAdminsMapper, PetAdmins> implements IPetAdminsService {

    @Autowired
    private PetAdminsMapper petAdminsMapper;

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 注册新用户
     * @return
     */
    @Override
    public Long register(AdminsSaveDTO adminsSaveDTO) {
        Long id = adminsSaveDTO.getId();
        if(id != null){
            // 修改
            PetAdmins petAdmins = petAdminsMapper.selectById(id);
            if(petAdmins == null){
                throw new PetShopException("没有找到对应的用户");
            }
            BeanUtils.copyProperties(adminsSaveDTO,petAdmins);
            petAdminsMapper.updateById(petAdmins);

            return id;
        }
        PetAdmins petAdmins = new PetAdmins();
        BeanUtils.copyProperties(adminsSaveDTO,petAdmins);
        long snowflakeNextId = IdUtil.getSnowflakeNextId();
        petAdmins.setId(snowflakeNextId);
        petAdmins.setIsDeleted(ENUM_DELETE_TYPE.UN_DELETE.getCode());
        petAdmins.setCreatedBy("system");
        petAdmins.setCreatedTime(LocalDateTimeUtil.now());
        petAdmins.setUpdatedBy("system");
        petAdmins.setUpdatedTime(LocalDateTimeUtil.now());
        return snowflakeNextId;
    }

    private boolean checkRegister(AdminsSaveDTO adminsSaveDTO){
        String count = adminsSaveDTO.getCount();
        List<PetAdmins> petAdmins = petAdminsMapper.selectList(new LambdaQueryWrapper<PetAdmins>().eq(PetAdmins::getCount, count).eq(PetAdmins::getIsDeleted,ENUM_DELETE_TYPE.UN_DELETE.getCode()));
        if(!CollectionUtils.isEmpty(petAdmins)){
            return false;
        }
        return true;
    }

    /**
     * 登录
     *
     * @param loginReqDTO
     * @return
     */
    @Override
    public LoginResVO login(LoginReqDTO loginReqDTO) {
        log.info("开始登录,获取用户输入信息:" + JSON.toJSONString(loginReqDTO));
        String username = loginReqDTO.getUsername();
        if(username == null || "".equals(username)){
            throw new PetShopException("没有用户名");
        }
        String password = loginReqDTO.getPassword();
        if(password == null || "".equals(password)){
            throw new PetShopException("没有密码");
        }

        // 根据用户输入信息去数据库中查询
        List<PetAdmins> petAdmins = petAdminsMapper.selectList(new LambdaQueryWrapper<PetAdmins>().eq(PetAdmins::getCount, username));
        if(CollectionUtils.isEmpty(petAdmins)){
            throw new PetShopException("没有找到该用户");
        }

        if(Boolean.TRUE.equals(redisTemplate.hasKey(redisKey))){
            Map<String,Object> redisMap = (Map<String, Object>) redisTemplate.opsForValue().get(redisKey);
            String code = (String) redisMap.get("code");
            log.info("获取验证码:{}",code);
            if(code == null || "".equals(code)){
                throw new PetShopException("验证码已过期");
            }
            if(!code.equals(loginReqDTO.getVerifyCode())){
                throw new PetShopException("验证码错误");
            }
        }else{
            throw new PetShopException("请先点击生成验证码");
        }

        PetAdmins admins = petAdmins.get(0);
        // 封装用户信息到jwt
        TokenVO tokenVO = new TokenVO();
        BeanUtils.copyProperties(admins,tokenVO);

        String token = JWTUtil.createToken(tokenVO);


        LoginResVO loginResVO = new LoginResVO();
        BeanUtils.copyProperties(tokenVO,loginResVO);
        loginResVO.setJwt(token);

//        Subject subject = SecurityUtils.getSubject();
//        AuthenticationToken authenticationToken = new JWTToken(token);
//        subject.login(authenticationToken);

        log.info("登录成功,返回前端:" + JSON.toJSONString(loginResVO));
        return loginResVO;
    }

    @Value("${login.redis.key}")
    private String redisKey;

    /**
     * 发送验证码
     * @param loginReqDTO
     * @return
     */
    @Override
    public String checkLogin(LoginReqDTO loginReqDTO){
        List<PetAdmins> petAdmins = petAdminsMapper.selectList(new LambdaQueryWrapper<PetAdmins>().eq(PetAdmins::getCount, loginReqDTO.getUsername()));
        if(CollectionUtils.isEmpty(petAdmins)){
            throw new PetShopException("该账户不存在");
        }
        PetAdmins admins = petAdmins.get(0);
        if(!admins.getPassword().equals(loginReqDTO.getPassword())){
            throw new PetShopException("密码错误");
        }
        // 随机生成验证码
        String code = VerifyCodeUtils.createVerifyCode();
        log.info("生成的验证码是:{}",code);

        Map<String,Object> redisMap = new LinkedHashMap<>();
        redisMap.put("code",code);

        // 将验证码存入缓存
        redisTemplate.opsForValue().set(redisKey,redisMap, Duration.ofMinutes(5L));
        // 将验证码通过短信发送给登录用户
        // TODO 封装用户信息,准备发送短信
        Map<String,Object> sendMap = new LinkedHashMap<>();
        sendMap.put("code",code);
        sendMap.put("name",admins.getName());
        sendMap.put("count",admins.getCount());

        return code;
    }
}
