package cn.youle.pet.shop.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.util.IdUtil;
import cn.youle.pet.shop.enums.ENUM_DELETE_TYPE;
import cn.youle.pet.shop.exception.PetShopException;
import cn.youle.pet.shop.pojo.dto.product.ProductDetailDTO;
import cn.youle.pet.shop.pojo.dto.product.ProductListDTO;
import cn.youle.pet.shop.pojo.dto.product.ProductSaveDTO;
import cn.youle.pet.shop.pojo.entity.PetProduct;
import cn.youle.pet.shop.mapper.PetProductMapper;
import cn.youle.pet.shop.pojo.vo.product.ProductDetailVO;
import cn.youle.pet.shop.pojo.vo.product.ProductListVO;
import cn.youle.pet.shop.service.IPetProductService;
import cn.youle.pet.shop.shiro.auth.TokenVO;
import cn.youle.pet.shop.utils.GetUserInfoUtils;
import cn.youle.pet.shop.utils.JWTUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 商品表 服务实现类
 * </p>
 *
 * @author wangrongli
 * @since 2023-09-19
 */
@Service
@Slf4j
public class PetProductServiceImpl extends ServiceImpl<PetProductMapper, PetProduct> implements IPetProductService {

    @Autowired
    private PetProductMapper petProductMapper;

    /**
     * 添加商品
     *
     * @param productSaveDTO 前端输入
     * @return 商品id
     */
    @Override
    public String create(ProductSaveDTO productSaveDTO, HttpServletRequest request) {
        log.info("=========================开始添加商品获取前端输入:{}=========================", JSON.toJSONString(productSaveDTO));
        if (productSaveDTO == null) {
            log.error("@@@@@@@@@@@@@@@@@@@@@前端输入为空@@@@@@@@@@@@@@@@@@@@@");
            throw new PetShopException("没有要添加的产品");
        }

        long snowflakeNextId = IdUtil.getSnowflakeNextId();
        PetProduct petProduct = new PetProduct();
        BeanUtils.copyProperties(productSaveDTO, petProduct);
        TokenVO userInfo = getUserInfo(request);
        petProduct.setId(snowflakeNextId);
        petProduct.setCreatedBy(userInfo.getName());
        petProduct.setCreatedTime(LocalDateTimeUtil.now());
        petProduct.setUpdatedBy(userInfo.getName());
        petProduct.setUpdatedTime(LocalDateTimeUtil.now());
        petProduct.setIsDeleted(ENUM_DELETE_TYPE.UN_DELETE.getCode());

        petProductMapper.insert(petProduct);

        log.info("=========================添加商品成功=========================");

        return snowflakeNextId + "";
    }

    /**
     * 修改商品
     *
     * @param productSaveDTO 前端输入
     * @return 商品id
     */
    @Override
    public String update(ProductSaveDTO productSaveDTO, HttpServletRequest request) {
        log.info("=========================开始更新商品获取前端输入:{}=========================", JSON.toJSONString(productSaveDTO));
        PetProduct petProduct = checkProduct(productSaveDTO.getProductId());
        BeanUtils.copyProperties(productSaveDTO, petProduct);
        TokenVO userInfo = getUserInfo(request);

        petProduct.setUpdatedBy(userInfo.getName());
        petProduct.setUpdatedTime(LocalDateTimeUtil.now());
        petProductMapper.updateById(petProduct);

        log.info("=========================更新商品成功=========================");

        return petProduct.getId() + "";
    }

    private PetProduct checkProduct(Long id) {
        if (id == null) {
            log.error("@@@@@@@@@@@@@@@@@@@@@id为空@@@@@@@@@@@@@@@@@@@@@");
            throw new PetShopException("没有对应的商品编码");
        }
        PetProduct petProduct = petProductMapper.selectById(id);
        if (petProduct == null) {
            log.error("@@@@@@@@@@@@@@@@@@@@@没有对应的商品@@@@@@@@@@@@@@@@@@@@@");
            throw new PetShopException("没有对应的商品");
        }
        return petProduct;
    }

    /**
     * 查询商品详情
     *
     * @param productDetailDTO 前端输入
     * @return 商品详情
     */
    @Override
    public ProductDetailVO detail(ProductDetailDTO productDetailDTO) {
        log.info("=========================开始查询商品详情获取前端输入:{}=========================", JSON.toJSONString(productDetailDTO));
        PetProduct petProduct = checkProduct(productDetailDTO.getId());
        ProductDetailVO productDetailVO = new ProductDetailVO();
        BeanUtils.copyProperties(petProduct, productDetailVO);

        log.info("=========================查询商品详情成功,即将返回:{}=========================", JSON.toJSONString(productDetailVO));

        return productDetailVO;
    }

    /**
     * 列表查询
     *
     * @param productListDTO 前端输入
     * @return 商品列表
     */
    @Override
    public ProductListVO list(ProductListDTO productListDTO) {
        log.info("=========================开始查询商品列表获取前端输入:{}=========================", JSON.toJSONString(productListDTO));
        ProductListVO productListVO = new ProductListVO();
        List<PetProduct> petProducts = petProductMapper.selectByListDto(productListDTO);
        if (CollectionUtils.isEmpty(petProducts)) {
            log.error("@@@@@@@@@@@@@@@@@@@@@没有对应的数据@@@@@@@@@@@@@@@@@@@@@");
            throw new PetShopException("没有对应的数据");
        }
        int page = (productListDTO.getCurrentPage() - 1) * productListDTO.getPageSize();
        List<PetProduct> collect = petProducts.stream().skip(page).limit(productListDTO.getPageSize()).collect(Collectors.toList());

        List<ProductDetailVO> productDetailVOList = new ArrayList<>();
        for (PetProduct petProduct : collect) {
            ProductDetailVO productDetailVO = new ProductDetailVO();
            BeanUtils.copyProperties(petProduct, productDetailVO);
            productDetailVOList.add(productDetailVO);
        }

        productListVO.setTotal(petProducts.size());
        productListVO.setProductDetailVOList(productDetailVOList);
        productListVO.setCurrentPage(productListDTO.getCurrentPage());
        productListVO.setPageSize(productListDTO.getPageSize());
        log.info("=========================查询商品列表成功,即将返回:{}=========================", JSON.toJSONString(productListDTO));

        return productListVO;
    }

    /**
     * 删除对应商品
     *
     * @param productDetailDTO 前端输入
     * @return 是否删除成功
     */
    @Override
    public int delete(ProductDetailDTO productDetailDTO) {
        log.info("===============准备开始删除获取前端输入:{}===============", JSON.toJSONString(productDetailDTO));
        PetProduct petProduct = checkProduct(productDetailDTO.getId());

        TokenVO userInfo = GetUserInfoUtils.getUserInfo();
        petProduct.setIsDeleted(productDetailDTO.getIsDeleted());
        petProduct.setUpdatedBy(userInfo.getName());
        petProduct.setUpdatedTime(LocalDateTimeUtil.now());

        int del = petProductMapper.updateById(petProduct);

        log.info("===============删除成功===============");

        return del;
    }

    /**
     * 修改商品状态
     *
     * @param productDetailDTO 前端输入
     */
    @Override
    public void updateStatus(ProductDetailDTO productDetailDTO) {
        log.info("=========================开始更新商品状态获取前端输入:{}=========================", JSON.toJSONString(productDetailDTO));
    }

    /**
     * 发送验证码
     *
     * @return
     */
    private String sendMessage() {
        return null;
    }

    private TokenVO getUserInfo(HttpServletRequest request) {
        String authorization = request.getHeader("Authorization");
        TokenVO tokenVO = JWTUtil.parseToken(authorization);
//        Subject subject = SecurityUtils.getSubject();
//        log.info("获取登录信息:{}",subject);
//        log.info("获取登录后信息:{}",subject.getPrincipal());
//        TokenVO tokenVO = (TokenVO) subject.getPrincipal();
        if (tokenVO == null) {
            throw new PetShopException("请先登录");
        }
        return tokenVO;
    }
}
