package cn.youle.pet.shop.shiro;

import cn.youle.pet.shop.exception.PetShopException;
import cn.youle.pet.shop.mapper.*;
import cn.youle.pet.shop.pojo.entity.*;
import cn.youle.pet.shop.shiro.auth.TokenVO;
import cn.youle.pet.shop.utils.JWTToken;
import cn.youle.pet.shop.utils.JWTUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.AuthenticationInfo;
import org.apache.shiro.authc.AuthenticationToken;
import org.apache.shiro.authc.SimpleAuthenticationInfo;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.authz.SimpleAuthorizationInfo;
import org.apache.shiro.realm.AuthorizingRealm;
import org.apache.shiro.subject.PrincipalCollection;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @projectName: pet-shop
 * @description:
 * @author: Administrator
 * @createTime: 2023/9/11 17:53
 */
@Slf4j
public class MyRealm extends AuthorizingRealm {

    @Autowired
    private PetAdminsMapper petAdminsMapper;

    @Autowired
    private PetRoleMapper petRoleMapper;

    @Autowired
    private PetPermissionMapper petPermissionMapper;

    @Autowired
    private PetAdminsRoleMapper petAdminsRoleMapper;

    @Autowired
    private PetRolePermissionMapper petRolePermissionMapper;

    @Override
    public boolean supports(AuthenticationToken token) {
        return token instanceof JWTToken;
    }

    /**
     * 授权
     *
     * @param principalCollection
     * @return
     */
    @Override
    protected AuthorizationInfo doGetAuthorizationInfo(PrincipalCollection principalCollection) {
        log.info("==================开始授权==================");
        // 获取登录用户信息
        Subject subject = SecurityUtils.getSubject();
        TokenVO tokenVO = (TokenVO) subject.getPrincipal();

        if (tokenVO == null) {
            throw new PetShopException("没有登录用户信息");
        }

        // 根据用户信息查询对应的角色
        List<PetAdmins> petAdmins = petAdminsMapper.selectList(new LambdaQueryWrapper<PetAdmins>().eq(PetAdmins::getCount, tokenVO.getCount()));
        if (CollectionUtils.isEmpty(petAdmins)) {
            throw new PetShopException("没有对应用户信息");
        }
        PetAdmins admins = petAdmins.get(0);
        // 根据用户id查找角色id
        List<PetAdminsRole> petAdminsRoles = petAdminsRoleMapper.selectList(new LambdaQueryWrapper<PetAdminsRole>().eq(PetAdminsRole::getAdminId, admins.getId()));
        if (CollectionUtils.isEmpty(petAdminsRoles)) {
            throw new PetShopException("没有对应用户的角色信息");
        }

        // 查询所有角色id
        List<Long> collect = petAdminsRoles.stream().map(PetAdminsRole::getRoleId).collect(Collectors.toList());

        // 将角色添加到下面的set中
        Set<String> roles = new HashSet<>();
        // 将对应的权限放入以下列表中
        Set<String> permissions = new HashSet<>();
        List<String> permissionIdList = new ArrayList<>();
        for (Long roleId : collect) {
            PetRole petRole = petRoleMapper.selectById(roleId);
            if (petRole == null) {
                throw new PetShopException("没有角色信息");
            }
            roles.add(petRole.getName());
            // 根据角色id查找对应的权限信息
            List<PetRolePermission> petRolePermissions = petRolePermissionMapper.selectList(new LambdaQueryWrapper<PetRolePermission>().eq(PetRolePermission::getRoleId, roleId));
            if (CollectionUtils.isEmpty(petRolePermissions)) {
                throw new PetShopException("没有权限信息");
            }

            // 获取所有权限的id集合
            for (PetRolePermission petRolePermission : petRolePermissions) {
                String permissionId = petRolePermission.getPermissionId();
                permissionIdList.add(permissionId);
            }
        }

        for (String permissionId : permissionIdList) {
            PetPermission petPermission = petPermissionMapper.selectById(Long.valueOf(permissionId));
            if (petPermission == null) {
                throw new PetShopException("没有权限信息");
            }
            permissions.add(petPermission.getValue());
        }

        SimpleAuthorizationInfo info = new SimpleAuthorizationInfo();
        log.info("==================授权成功==================");
//        info.addStringPermissions(permissions);
//        info.addRoles(roles);

        return info;
    }

    /**
     * 认证
     */
    @Override
    protected AuthenticationInfo doGetAuthenticationInfo(AuthenticationToken authenticationToken) throws AuthenticationException {
        log.info("==================开始认证==================");
        if (authenticationToken == null) {
            // 报错
            throw new AuthenticationException("没有需要认证的数据");
        }
        log.info("用户输入的内容:{},{}",authenticationToken.getCredentials(),authenticationToken.getPrincipal());
        String token = (String) authenticationToken.getCredentials();
        if (token == null) {
            throw new AuthenticationException("token为空");
        }
        // 解析token
        TokenVO tokenVO = JWTUtil.parseToken(token);
        if (tokenVO == null) {
            throw new AuthenticationException("解析token出错");
        }
        // 解密获取username,用于和数据库进行对比
        String count = tokenVO.getCount();
        List<PetAdmins> petAdmins = petAdminsMapper.selectList(new LambdaQueryWrapper<PetAdmins>().eq(PetAdmins::getCount, count));
        if (CollectionUtils.isEmpty(petAdmins)) {
            // 报错
            throw new AuthenticationException("没有该用户");
        }

        SimpleAuthenticationInfo info = new SimpleAuthenticationInfo(tokenVO, token, getName());

        log.info("==================认证成功==================");
        return info;
    }
}
